@@ -132,7 +132,7 @@ static int extend_file(struct thread_data *td, struct fio_file *f)
 			goto err;
 		}
 	}
-	if (td->o.fill_device) {
+	if (td->o.fill_device && !td_write(td)) {
 		fio_file_clear_size_known(f);
 		if (td_io_get_file_size(td, f))
 			goto err;
@@ -246,6 +246,7 @@ static int bdev_size(struct thread_data *td, struct fio_file *f)
 	}
 
 	f->real_file_size = bytes;
+	td->io_ops->close_file(td, f);
 	return 0;
 err:
 	td->io_ops->close_file(td, f);
@@ -290,6 +291,9 @@ static int __file_invalidate_cache(struct thread_data *td, struct fio_file *f,
 	if (off == -1ULL)
 		off = f->file_offset;
 
+	if (len == -1ULL || off == -1ULL)
+		return 0;
+
 	dprint(FD_IO, "invalidate cache %s: %llu/%llu\n", f->file_name, off,
 								len);
 
@@ -723,7 +727,9 @@ void close_and_free_files(struct thread_data *td)
 			unlink(f->file_name);
 		}
 
-		td_io_close_file(td, f);
+		if (fio_file_open(f))
+			td_io_close_file(td, f);
+
 		remove_file_hash(f);
 
 		sfree(f->file_name);
@@ -844,8 +850,10 @@ int put_file(struct thread_data *td, struct fio_file *f)
 
 	dprint(FD_FILE, "put file %s, ref=%d\n", f->file_name, f->references);
 
-	if (!fio_file_open(f))
+	if (!fio_file_open(f)) {
+		assert(f->fd == -1);
 		return 0;
+	}
 
 	assert(f->references);
 	if (--f->references)
@@ -862,6 +870,7 @@ int put_file(struct thread_data *td, struct fio_file *f)
 
 	td->nr_open_files--;
 	fio_file_clear_open(f);
+	assert(f->fd == -1);
 	return ret;
 }
 
@@ -1001,6 +1010,7 @@ void dup_files(struct thread_data *td, struct thread_data *org)
 			log_err("fio: smalloc OOM\n");
 			assert(0);
 		}
+		__f->fd = -1;
 	
 		if (f->file_name) {
 			__f->file_name = smalloc_strdup(f->file_name);
